@@ -1,6 +1,9 @@
 /*
- * (C) 2012 by Pablo Neira Ayuso <pablo@gnumonks.org>
+ * (C) 2012-2017 by Pablo Neira Ayuso <pablo@gnumonks.org>
  * (C) 2012 by On Waves ehf <http://www.on-waves.com>
+ * (C) 2015-2017 by sysmocom - s.f.m.c. GmbH
+ *
+ * SPDX-License-Identifier: GPL-2.0+
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,6 +16,7 @@
 
 #include <osmocom/core/msgb.h>
 #include <osmocom/core/timer.h>
+#include <osmocom/core/timer_compat.h>
 #include <osmocom/core/select.h>
 #include <osmocom/core/talloc.h>
 #include <osmocom/core/logging.h>
@@ -82,16 +86,21 @@ next:
 		case OSMUX_FT_VOICE_AMR:
 			break;
 		case OSMUX_FT_DUMMY:
-			msgb_pull(msg, osmux_ft_dummy_size(osmuxh->amr_ft,
-							   osmuxh->ctr + 1));
+			len = osmux_ft_dummy_size(osmuxh->amr_ft, osmuxh->ctr + 1);
+			if (msgb_length(msg) < len) {
+				LOGP(DLMUX, LOGL_ERROR, "Discarding bad Dummy FT: %s\n",
+					osmo_hexdump(msg->data, msgb_length(msg)));
+				return NULL;
+			}
+			msgb_pull(msg, len);
 			goto next;
 		default:
-			LOGP(DLMIB, LOGL_ERROR, "Discarding unsupported Osmux FT %d\n",
+			LOGP(DLMUX, LOGL_ERROR, "Discarding unsupported Osmux FT %d\n",
 			     osmuxh->ft);
 			return NULL;
 		}
 		if (!osmo_amr_ft_valid(osmuxh->amr_ft)) {
-			LOGP(DLMIB, LOGL_ERROR, "Discarding bad AMR FT %d\n",
+			LOGP(DLMUX, LOGL_ERROR, "Discarding bad AMR FT %d\n",
 			     osmuxh->amr_ft);
 			return NULL;
 		}
@@ -99,15 +108,16 @@ next:
 		len = osmo_amr_bytes(osmuxh->amr_ft) * (osmuxh->ctr+1) +
 			sizeof(struct osmux_hdr);
 
-		if (len > msg->len) {
-			LOGP(DLMIB, LOGL_ERROR, "Discarding malformed "
-						"OSMUX message\n");
+		if (msgb_length(msg) < len) {
+			LOGP(DLMUX, LOGL_ERROR,
+				"Discarding malformed OSMUX message: %s\n",
+				osmo_hexdump(msg->data, msgb_length(msg)));
 			return NULL;
 		}
 
 		msgb_pull(msg, len);
 	} else if (msg->len > 0) {
-		LOGP(DLMIB, LOGL_ERROR,
+		LOGP(DLMUX, LOGL_ERROR,
 			"remaining %d bytes, broken osmuxhdr?\n", msg->len);
 	}
 
@@ -118,9 +128,11 @@ static struct msgb *
 osmux_rebuild_rtp(struct osmux_out_handle *h, struct osmux_hdr *osmuxh,
 		  void *payload, int payload_len, bool first_pkt)
 {
-	struct msgb *out_msg;
+	struct msgb *prev_msg, *out_msg;
+	struct timespec *prev_ts, *out_ts;
 	struct rtp_hdr *rtph;
 	struct amr_hdr *amrh;
+	struct timespec delta = { .tv_sec = 0, .tv_nsec = DELTA_RTP_MSG*1000 };
 
 	out_msg = msgb_alloc(sizeof(struct rtp_hdr) +
 			     sizeof(struct amr_hdr) +
@@ -139,8 +151,14 @@ osmux_rebuild_rtp(struct osmux_out_handle *h, struct osmux_hdr *osmuxh,
 	rtph->timestamp = htonl(h->rtp_timestamp);
 	rtph->sequence = htons(h->rtp_seq);
 	rtph->ssrc = htonl(h->rtp_ssrc);
-	/* rtp packet with the marker bit is always warranted to be the first one */
-	rtph->marker = first_pkt && osmuxh->rtp_m;
+	/* rtp packet with the marker bit is always guaranteed to be the first
+	 * one. We want to notify with marker in 2 scenarios:
+	 * 1- Sender told us through osmux frame rtp_m.
+	 * 2- Sntermediate osmux frame lost (seq gap), otherwise rtp receiver only sees
+	 *    steady increase of delay
+	 */
+	rtph->marker = first_pkt &&
+			(osmuxh->rtp_m || (osmuxh->seq != h->osmux_seq_ack + 1));
 
 	msgb_put(out_msg, sizeof(struct rtp_hdr));
 
@@ -161,6 +179,15 @@ osmux_rebuild_rtp(struct osmux_out_handle *h, struct osmux_hdr *osmuxh,
 	h->rtp_seq++;
 	h->rtp_timestamp += DELTA_RTP_TIMESTAMP;
 
+	out_ts = ((struct timespec *)&((out_msg)->cb[0]));
+	if (first_pkt || llist_empty(&h->list)) {
+		osmo_clock_gettime(CLOCK_MONOTONIC, out_ts);
+	} else {
+		prev_msg = llist_last_entry(&h->list, struct msgb, list);
+		prev_ts = ((struct timespec *)&((prev_msg)->cb[0]));
+		timespecadd(prev_ts, &delta, out_ts);
+	}
+
 	return out_msg;
 }
 
@@ -192,14 +219,122 @@ int osmux_xfrm_output(struct osmux_hdr *osmuxh, struct osmux_out_handle *h,
 
 			osmo_rtp_snprintf(buf, sizeof(buf), msg);
 			buf[sizeof(buf)-1] = '\0';
-			LOGP(DLMIB, LOGL_DEBUG, "to BTS: %s\n", buf);
+			LOGP(DLMUX, LOGL_DEBUG, "to BTS: %s\n", buf);
 		}
 #endif
 		llist_add_tail(&msg->list, list);
 	}
+
+	/* Update last seen seq number: */
+	h->osmux_seq_ack = osmuxh->seq;
+
 	return i;
 }
 
+static void osmux_xfrm_output_trigger(void *data)
+{
+	struct osmux_out_handle *h = data;
+	struct timespec delay_ts, now;
+	struct msgb *msg, *next;
+
+	llist_for_each_entry_safe(msg, next, &h->list, list) {
+		osmo_clock_gettime(CLOCK_MONOTONIC, &now);
+		struct timespec *msg_ts = ((struct timespec *)&((msg)->cb[0]));
+		if (timespeccmp(msg_ts, &now, >)) {
+			timespecsub(msg_ts, &now, &delay_ts);
+			osmo_timer_schedule(&h->timer,
+				delay_ts.tv_sec, delay_ts.tv_nsec / 1000);
+			return;
+		}
+
+		/* Transmit the rtp packet */
+		llist_del(&msg->list);
+		if (h->tx_cb)
+			h->tx_cb(msg, h->data);
+		else
+			msgb_free(msg);
+	}
+}
+
+/*! \brief Generate RTP packets from osmux frame AMR payload set and schedule
+ *         them for transmission at appropiate time.
+ *  \param[in] h the osmux out handle handling a specific CID
+ *  \param[in] osmuxh Buffer pointing to osmux frame header structure and AMR payload
+ *  \return Number of generated RTP packets
+ *
+ * The osmux frame passed to this function must be of the type OSMUX_FT_VOICE_AMR.
+ * The generated RTP packets are kept into h's internal list and sent to the
+ * callback configured through osmux_xfrm_output_set_tx_cb when are ready to be
+ * transmitted according to schedule.
+ */
+int osmux_xfrm_output_sched(struct osmux_out_handle *h, struct osmux_hdr *osmuxh)
+{
+	struct timespec now, *msg_ts;
+	struct msgb *msg;
+	int i;
+	bool was_empty = llist_empty(&h->list);
+
+	if (!was_empty) {
+		/* If we received new data it means we are behind schedule and
+		 * we should flush all previous quickly */
+		osmo_clock_gettime(CLOCK_MONOTONIC, &now);
+		llist_for_each_entry(msg, &h->list, list) {
+			msg_ts = ((struct timespec *)&((msg)->cb[0]));
+			*msg_ts = now;
+		}
+		osmo_timer_schedule(&h->timer, 0, 0);
+	}
+
+	for (i=0; i<osmuxh->ctr+1; i++) {
+		struct rtp_hdr *rtph;
+
+		msg = osmux_rebuild_rtp(h, osmuxh,
+					osmux_get_payload(osmuxh) +
+					i * osmo_amr_bytes(osmuxh->amr_ft),
+					osmo_amr_bytes(osmuxh->amr_ft), !i);
+		if (msg == NULL)
+			continue;
+
+		rtph = osmo_rtp_get_hdr(msg);
+		if (rtph == NULL)
+			continue;
+
+		llist_add_tail(&msg->list, &h->list);
+	}
+
+	/* Update last seen seq number: */
+	h->osmux_seq_ack = osmuxh->seq;
+
+	/* In case list is still empty after parsing messages, no need to rearm */
+	if(was_empty && !llist_empty(&h->list))
+		osmux_xfrm_output_trigger(h);
+	return i;
+}
+
+/*! \brief Flush all scheduled RTP packets still pending to be transmitted
+ *  \param[in] h the osmux out handle to flush
+ *
+ * This function will immediately call the transmit callback for all queued RTP
+ * packets, making sure the list ends up empty. It will also stop all internal
+ * timers to make sure the osmux_out_handle can be dropped or re-used by calling
+ * osmux_xfrm_output on it.
+ */
+void osmux_xfrm_output_flush(struct osmux_out_handle *h)
+{
+	struct msgb *msg, *next;
+
+	if (osmo_timer_pending(&h->timer))
+		osmo_timer_del(&h->timer);
+
+	llist_for_each_entry_safe(msg, next, &h->list, list) {
+		llist_del(&msg->list);
+		if (h->tx_cb)
+			h->tx_cb(msg, h->data);
+		else
+			msgb_free(msg);
+	}
+}
+
 struct osmux_batch {
 	struct osmo_timer_list	timer;
 	struct osmux_hdr	*osmuxh;
@@ -221,7 +356,7 @@ struct osmux_circuit {
 static int osmux_batch_enqueue(struct msgb *msg, struct osmux_circuit *circuit,
 				uint8_t batch_factor)
 {
-	/* Too many messages per batch, discard it. The counter field of the
+	/* Validate amount of messages per batch. The counter field of the
 	 * osmux header is just 3 bits long, so make sure it doesn't overflow.
 	 */
 	if (circuit->nmsgs >= batch_factor || circuit->nmsgs >= 8) {
@@ -231,7 +366,7 @@ static int osmux_batch_enqueue(struct msgb *msg, struct osmux_circuit *circuit,
 		if (rtph == NULL)
 			return -1;
 
-		LOGP(DLMIB, LOGL_ERROR, "too many messages for this RTP "
+		LOGP(DLMUX, LOGL_DEBUG, "Batch is full for RTP "
 					"ssrc=%u\n", rtph->ssrc);
 		return -1;
 	}
@@ -289,7 +424,7 @@ static int osmux_batch_put(struct osmux_batch *batch,
 		batch->osmuxh = osmuxh;
 	} else {
 		if (batch->osmuxh->ctr == 0x7) {
-			LOGP(DLMIB, LOGL_ERROR, "cannot add msg=%p, "
+			LOGP(DLMUX, LOGL_ERROR, "cannot add msg=%p, "
 			     "too many messages for this RTP ssrc=%u\n",
 			     state->msg, state->rtph->ssrc);
 			return 0;
@@ -351,12 +486,12 @@ static struct msgb *osmux_build_batch(struct osmux_batch *batch,
 	struct osmux_circuit *circuit;
 
 #ifdef DEBUG_MSG
-	LOGP(DLMIB, LOGL_DEBUG, "Now building batch\n");
+	LOGP(DLMUX, LOGL_DEBUG, "Now building batch\n");
 #endif
 
 	batch_msg = msgb_alloc(batch_size, "osmux");
 	if (batch_msg == NULL) {
-		LOGP(DLMIB, LOGL_ERROR, "Not enough memory\n");
+		LOGP(DLMUX, LOGL_ERROR, "Not enough memory\n");
 		return NULL;
 	}
 
@@ -384,7 +519,7 @@ static struct msgb *osmux_build_batch(struct osmux_batch *batch,
 
 			osmo_rtp_snprintf(buf, sizeof(buf), cur);
 			buf[sizeof(buf)-1] = '\0';
-			LOGP(DLMIB, LOGL_DEBUG, "to BSC-NAT: %s\n", buf);
+			LOGP(DLMUX, LOGL_DEBUG, "to BSC-NAT: %s\n", buf);
 #endif
 
 			state.rtph = osmo_rtp_get_hdr(cur);
@@ -393,7 +528,7 @@ static struct msgb *osmux_build_batch(struct osmux_batch *batch,
 
 			if (ctr == 0) {
 #ifdef DEBUG_MSG
-				LOGP(DLMIB, LOGL_DEBUG, "add osmux header\n");
+				LOGP(DLMUX, LOGL_DEBUG, "add osmux header\n");
 #endif
 				state.add_osmux_hdr = 1;
 			}
@@ -414,7 +549,7 @@ void osmux_xfrm_input_deliver(struct osmux_in_handle *h)
 	struct osmux_batch *batch = (struct osmux_batch *)h->internal_data;
 
 #ifdef DEBUG_MSG
-	LOGP(DLMIB, LOGL_DEBUG, "invoking delivery function\n");
+	LOGP(DLMUX, LOGL_DEBUG, "invoking delivery function\n");
 #endif
 	batch_msg = osmux_build_batch(batch, h->batch_size, h->batch_factor);
 	if (!batch_msg)
@@ -437,7 +572,7 @@ static void osmux_batch_timer_expired(void *data)
 	struct osmux_in_handle *h = data;
 
 #ifdef DEBUG_MSG
-	LOGP(DLMIB, LOGL_DEBUG, "osmux_batch_timer_expired\n");
+	LOGP(DLMUX, LOGL_DEBUG, "osmux_batch_timer_expired\n");
 #endif
 	osmux_xfrm_input_deliver(h);
 }
@@ -459,7 +594,7 @@ static int osmux_rtp_amr_payload_len(struct msgb *msg, struct rtp_hdr *rtph)
 
 	/* The AMR payload does not fit with what we expect */
 	if (osmo_amr_bytes(amrh->ft) != amr_payload_len) {
-		LOGP(DLMIB, LOGL_ERROR,
+		LOGP(DLMUX, LOGL_ERROR,
 		     "Bad AMR frame, expected %zd bytes, got %d bytes\n",
 		     osmo_amr_bytes(amrh->ft), amr_len);
 		return -1;
@@ -519,7 +654,7 @@ static void osmux_replay_lost_packets(struct osmux_circuit *circuit,
 			break;
 		}
 
-		LOGP(DLMIB, LOGL_ERROR, "adding cloned RTP\n");
+		LOGP(DLMUX, LOGL_ERROR, "adding cloned RTP\n");
 	}
 }
 
@@ -543,13 +678,13 @@ osmux_batch_add_circuit(struct osmux_batch *batch, int ccid, int dummy,
 
 	circuit = osmux_batch_find_circuit(batch, ccid);
 	if (circuit != NULL) {
-		LOGP(DLMIB, LOGL_ERROR, "circuit %u already exists!\n", ccid);
+		LOGP(DLMUX, LOGL_ERROR, "circuit %u already exists!\n", ccid);
 		return NULL;
 	}
 
 	circuit = talloc_zero(osmux_ctx, struct osmux_circuit);
 	if (circuit == NULL) {
-		LOGP(DLMIB, LOGL_ERROR, "OOM on circuit %u\n", ccid);
+		LOGP(DLMUX, LOGL_ERROR, "OOM on circuit %u\n", ccid);
 		return NULL;
 	}
 
@@ -623,7 +758,7 @@ osmux_batch_add(struct osmux_batch *batch, uint32_t batch_factor, struct msgb *m
 
 		/* Already exists message with this sequence, skip */
 		if (rtph2->sequence == rtph->sequence) {
-			LOGP(DLMIB, LOGL_ERROR, "already exists "
+			LOGP(DLMUX, LOGL_ERROR, "already exists "
 				"message with seq=%u, skip it\n",
 				rtph->sequence);
 			return -1;
@@ -637,7 +772,7 @@ osmux_batch_add(struct osmux_batch *batch, uint32_t batch_factor, struct msgb *m
 		return 1;
 
 #ifdef DEBUG_MSG
-	LOGP(DLMIB, LOGL_DEBUG, "adding msg with ssrc=%u to batch\n",
+	LOGP(DLMUX, LOGL_DEBUG, "adding msg with ssrc=%u to batch\n",
 		rtph->ssrc);
 #endif
 
@@ -646,7 +781,7 @@ osmux_batch_add(struct osmux_batch *batch, uint32_t batch_factor, struct msgb *m
 
 	if (batch->nmsgs == 0) {
 #ifdef DEBUG_MSG
-		LOGP(DLMIB, LOGL_DEBUG, "osmux start timer batch\n");
+		LOGP(DLMUX, LOGL_DEBUG, "osmux start timer batch\n");
 #endif
 		osmo_timer_schedule(&batch->timer, 0,
 				    batch_factor * DELTA_RTP_MSG);
@@ -734,7 +869,23 @@ void osmux_xfrm_input_init(struct osmux_in_handle *h)
 
 	h->internal_data = (void *)batch;
 
-	LOGP(DLMIB, LOGL_DEBUG, "initialized osmux input converter\n");
+	LOGP(DLMUX, LOGL_DEBUG, "initialized osmux input converter\n");
+}
+
+/*! \brief Set transmission callback to call when a generated RTP packet is to be transmitted
+ *  \param[in] h the osmux out handle handling a specific CID
+ *  \param[in] osmuxh Buffer pointing to osmux frame header structure and AMR payload
+ *  \return Number of generated RTP packets
+ *
+ * This Function sets the callback called by the interal timer set by
+ * osmux_xfrm_out_sched function.
+ */
+void osmux_xfrm_output_set_tx_cb(struct osmux_out_handle *h,
+				void (*tx_cb)(struct msgb *msg, void *data),
+				void *data)
+{
+	h->tx_cb = tx_cb;
+	h->data = data;
 }
 
 int osmux_xfrm_input_open_circuit(struct osmux_in_handle *h, int ccid,
@@ -789,7 +940,7 @@ static void osmux_tx_cb(void *data)
 	osmo_gettimeofday(&now, NULL);
 	timersub(&now, &h->start, &diff);
 	timersub(&diff,&h->when, &diff);
-	LOGP(DLMIB, LOGL_DEBUG, "we are lagging %lu.%.6lu in scheduled "
+	LOGP(DLMUX, LOGL_DEBUG, "we are lagging %lu.%.6lu in scheduled "
 		"transmissions\n", diff.tv_sec, diff.tv_usec);
 #endif
 
@@ -839,7 +990,7 @@ osmux_tx_sched(struct llist_head *list,
 	llist_for_each_entry_safe(cur, tmp, list, list) {
 
 #ifdef DEBUG_MSG
-		LOGP(DLMIB, LOGL_DEBUG, "scheduled transmision in %lu.%6lu "
+		LOGP(DLMUX, LOGL_DEBUG, "scheduled transmision in %lu.%6lu "
 			"seconds, msg=%p\n", when.tv_sec, when.tv_usec, cur);
 #endif
 		llist_del(&cur->list);
@@ -850,9 +1001,12 @@ osmux_tx_sched(struct llist_head *list,
 
 void osmux_xfrm_output_init(struct osmux_out_handle *h, uint32_t rtp_ssrc)
 {
+	memset(h, 0, sizeof(*h));
 	h->rtp_seq = (uint16_t)random();
 	h->rtp_timestamp = (uint32_t)random();
 	h->rtp_ssrc = rtp_ssrc;
+	INIT_LLIST_HEAD(&h->list);
+	osmo_timer_setup(&h->timer, osmux_xfrm_output_trigger, h);
 }
 
 #define SNPRINTF_BUFFER_SIZE(ret, remain, offset)	\
@@ -926,7 +1080,7 @@ int osmux_snprintf(char *buf, size_t size, struct msgb *msg)
 
 	while (msg_len > 0) {
 		if (msg_len < sizeof(struct osmux_hdr)) {
-			LOGP(DLMIB, LOGL_ERROR,
+			LOGP(DLMUX, LOGL_ERROR,
 			     "No room for OSMUX header: only %d bytes\n",
 			     msg_len);
 			return -1;
@@ -947,7 +1101,7 @@ int osmux_snprintf(char *buf, size_t size, struct msgb *msg)
 		case OSMUX_FT_DUMMY:
 		case OSMUX_FT_VOICE_AMR:
 			if (!osmo_amr_ft_valid(osmuxh->amr_ft)) {
-				LOGP(DLMIB, LOGL_ERROR, "Bad AMR FT %d, skipping\n",
+				LOGP(DLMUX, LOGL_ERROR, "Bad AMR FT %d, skipping\n",
 				     osmuxh->amr_ft);
 				return -1;
 			}
@@ -955,7 +1109,7 @@ int osmux_snprintf(char *buf, size_t size, struct msgb *msg)
 			payload_len = osmux_get_payload_len(osmuxh);
 
 			if (msg_len < payload_len) {
-				LOGP(DLMIB, LOGL_ERROR,
+				LOGP(DLMUX, LOGL_ERROR,
 				     "No room for OSMUX payload: only %d bytes\n",
 				     msg_len);
 				return -1;
@@ -972,7 +1126,7 @@ int osmux_snprintf(char *buf, size_t size, struct msgb *msg)
 			msg_len -= payload_len;
 			break;
 		default:
-			LOGP(DLMIB, LOGL_ERROR, "Unknown OSMUX ft value %d\n",
+			LOGP(DLMUX, LOGL_ERROR, "Unknown OSMUX ft value %d\n",
 			     osmuxh->ft);
 			return -1;
 		}
