@@ -1,3 +1,24 @@
+/* (C) 2011 by Pablo Neira Ayuso <pablo@gnumonks.org>
+ * (C) 2015-2016 by Harald Welte <laforge@gnumonks.org>
+ * All Rights Reserved.
+ *
+ * SPDX-License-Identifier: GPL-2.0+
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ */
+
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -409,6 +430,7 @@ void osmo_stream_cli_destroy(struct osmo_stream_cli *cli)
 {
 	osmo_stream_cli_close(cli);
 	osmo_timer_del(&cli->timer);
+	msgb_queue_free(&cli->tx_queue);
 	talloc_free(cli);
 }
 
@@ -429,9 +451,9 @@ int osmo_stream_cli_open2(struct osmo_stream_cli *cli, int reconnect)
 	ret = osmo_sock_init2(AF_INET, SOCK_STREAM, cli->proto,
 			      cli->local_addr, cli->local_port,
 			      cli->addr, cli->port,
-			      OSMO_SOCK_F_CONNECT|OSMO_SOCK_F_BIND);
+			      OSMO_SOCK_F_CONNECT|OSMO_SOCK_F_BIND|OSMO_SOCK_F_NONBLOCK);
 	if (ret < 0) {
-		if (reconnect && errno == ECONNREFUSED)
+		if (reconnect)
 			osmo_stream_cli_reconnect(cli);
 		return ret;
 	}
@@ -714,9 +736,13 @@ int osmo_stream_srv_link_open(struct osmo_stream_srv_link *link)
 {
 	int ret;
 
-	/* we are reconfiguring this socket, close existing first. */
-	if ((link->flags & OSMO_STREAM_SRV_F_RECONF) && link->ofd.fd >= 0)
+	if (link->ofd.fd >= 0) {
+		/* No reconfigure needed for existing socket, we are fine */
+		if (!(link->flags & OSMO_STREAM_SRV_F_RECONF))
+			return 0;
+		/* we are reconfiguring this socket, close existing first. */
 		osmo_stream_srv_link_close(link);
+	}
 
 	link->flags &= ~OSMO_STREAM_SRV_F_RECONF;
 
@@ -746,6 +772,8 @@ void osmo_stream_srv_link_close(struct osmo_stream_srv_link *link)
 	link->ofd.fd = -1;
 }
 
+#define OSMO_STREAM_SRV_F_FLUSH_DESTROY	(1 << 0)
+
 struct osmo_stream_srv {
 	struct osmo_stream_srv_link	*srv;
         struct osmo_fd                  ofd;
@@ -753,12 +781,18 @@ struct osmo_stream_srv {
         int (*closed_cb)(struct osmo_stream_srv *peer);
         int (*cb)(struct osmo_stream_srv *peer);
         void                            *data;
+	int				flags;
 };
 
 static void osmo_stream_srv_read(struct osmo_stream_srv *conn)
 {
 	LOGP(DLINP, LOGL_DEBUG, "message received\n");
 
+	if (conn->flags & OSMO_STREAM_SRV_F_FLUSH_DESTROY) {
+		LOGP(DLINP, LOGL_DEBUG, "Connection is being flushed and closed; ignoring received message\n");
+		return;
+	}
+
 	if (conn->cb)
 		conn->cb(conn);
 
@@ -803,6 +837,9 @@ static void osmo_stream_srv_write(struct osmo_stream_srv *conn)
 		LOGP(DLINP, LOGL_ERROR, "error to send\n");
 	}
 	msgb_free(msg);
+
+	if (llist_empty(&conn->tx_queue) && (conn->flags & OSMO_STREAM_SRV_F_FLUSH_DESTROY))
+		osmo_stream_srv_destroy(conn);
 }
 
 static int osmo_stream_srv_cb(struct osmo_fd *ofd, unsigned int what)
@@ -854,6 +891,16 @@ osmo_stream_srv_create(void *ctx, struct osmo_stream_srv_link *link,
 	return conn;
 }
 
+/*! \brief Prepare to send out all pending messages on the connection's Tx queue
+ *  and then automatically destroy the stream with osmo_stream_srv_destroy().
+ *  This function disables queuing of new messages on the connection and also
+ *  disables reception of new messages on the connection.
+ *  \param[in] conn Stream Server to modify */
+void osmo_stream_srv_set_flush_and_destroy(struct osmo_stream_srv *conn)
+{
+	conn->flags |= OSMO_STREAM_SRV_F_FLUSH_DESTROY;
+}
+
 /*! \brief Set application private data of the stream server
  *  \param[in] conn Stream Server to modify
  *  \param[in] data User-specific data (available in call-back functions) */
@@ -891,7 +938,9 @@ struct osmo_stream_srv_link *osmo_stream_srv_get_master(struct osmo_stream_srv *
 
 /*! \brief Destroy given Stream Server
  *  This function closes the Stream Server socket, unregisters from
- *  select loop and de-allocates associated memory.
+ *  select loop, invokes the connection's closed_cb() callback to allow API
+ *  users to clean up any associated state they have for this connection,
+ *  and then de-allocates associated memory.
  *  \param[in] conn Stream Server to be destroyed */
 void osmo_stream_srv_destroy(struct osmo_stream_srv *conn)
 {
@@ -899,6 +948,7 @@ void osmo_stream_srv_destroy(struct osmo_stream_srv *conn)
 	osmo_fd_unregister(&conn->ofd);
 	if (conn->closed_cb)
 		conn->closed_cb(conn);
+	msgb_queue_free(&conn->tx_queue);
 	talloc_free(conn);
 }
 
@@ -907,6 +957,11 @@ void osmo_stream_srv_destroy(struct osmo_stream_srv *conn)
  *  \param[in] msg Message buffer to enqueue in transmit queue */
 void osmo_stream_srv_send(struct osmo_stream_srv *conn, struct msgb *msg)
 {
+	if (conn->flags & OSMO_STREAM_SRV_F_FLUSH_DESTROY) {
+		LOGP(DLINP, LOGL_DEBUG, "Connection is being flushed and closed; ignoring new outgoing message\n");
+		return;
+	}
+
 	msgb_enqueue(&conn->tx_queue, msg);
 	conn->ofd.when |= BSC_FD_WRITE;
 }
