@@ -103,7 +103,6 @@ static struct osmux_out_handle h_output;
 static void osmux_deliver(struct msgb *batch_msg, void *data)
 {
 	struct osmux_hdr *osmuxh;
-	LLIST_HEAD(list);
 	char buf[2048];
 
 	osmux_snprintf(buf, sizeof(buf), batch_msg);
@@ -112,10 +111,8 @@ static void osmux_deliver(struct msgb *batch_msg, void *data)
 	/* For each OSMUX message, extract the RTP messages and put them
 	 * in a list. Then, reconstruct transmission timing.
 	 */
-	while((osmuxh = osmux_xfrm_output_pull(batch_msg)) != NULL) {
-		osmux_xfrm_output(osmuxh, &h_output, &list);
-		osmux_tx_sched(&list, tx_cb, NULL);
-	}
+	while((osmuxh = osmux_xfrm_output_pull(batch_msg)) != NULL)
+		osmux_xfrm_output_sched(&h_output, osmuxh);
 	msgb_free(batch_msg);
 }
 
@@ -152,7 +149,8 @@ static void osmux_test_marker(int ccid) {
 			memcpy(msg->data, rtp_pkt, sizeof(rtp_pkt));
 			cpy_rtph = (struct rtp_hdr *) msgb_put(msg, sizeof(rtp_pkt));
 
-			if ((i+j) % 7 == 0) {
+			/* first condition guarantees that 1st packet per stream contains M bit set. */
+			if (i == 0 || (i+j) % 7 == 0) {
 				cpy_rtph->marker = 1;
 				mark_pkts++;
 			}
@@ -175,7 +173,7 @@ static void osmux_test_marker(int ccid) {
 	}
 
 	if (mark_pkts) {
-		fprintf(stdout, "RTP M bit (marker) mismatch! %d\n", mark_pkts);
+		fprintf(stdout, "osmux_test_marker: RTP M bit (marker) mismatch! %d\n", mark_pkts);
 		exit(EXIT_FAILURE);
 	}
 }
@@ -183,6 +181,7 @@ static void osmux_test_marker(int ccid) {
 static void osmux_test_loop(int ccid)
 {
 	struct rtp_hdr *rtph = (struct rtp_hdr *)rtp_pkt;
+	struct rtp_hdr *cpy_rtph;
 	struct msgb *msg;
 	int i, j, k = 0;
 	char buf[1024];
@@ -194,11 +193,16 @@ static void osmux_test_loop(int ccid)
 			exit(EXIT_FAILURE);
 
 		memcpy(msg->data, rtp_pkt, sizeof(rtp_pkt));
-		msgb_put(msg, sizeof(rtp_pkt));
+		cpy_rtph = (struct rtp_hdr *) msgb_put(msg, sizeof(rtp_pkt));
 
 		seq = ntohs(rtph->sequence);
 		seq++;
 		rtph->sequence = htons(seq);
+		if (i < 3) {
+			/* Mark 1 rtp packet of each stream */
+			cpy_rtph->marker = 1;
+			mark_pkts++;
+		}
 
 		osmo_rtp_snprintf(buf, sizeof(buf), msg);
 		fprintf(stderr, "adding to ccid=%u %s\n", (i % 2) + ccid, buf);
@@ -239,7 +243,7 @@ static void osmux_test_loop(int ccid)
 	}
 
 	if (mark_pkts) {
-		fprintf(stdout, "RTP M bit (marker) mismatch! %d\n", mark_pkts);
+		fprintf(stdout, "osmux_test_loop: RTP M bit (marker) mismatch! %d\n", mark_pkts);
 		exit(EXIT_FAILURE);
 	}
 }
@@ -260,10 +264,13 @@ int main(void)
 #endif
 
 	/* This test doesn't use it, but osmux requires it internally. */
-	osmo_init_logging(&osmux_test_log_info);
+	void *tall_ctx = talloc_named_const(NULL, 1, "Root context");
+	msgb_talloc_ctx_init(tall_ctx, 0);
+	osmo_init_logging2(tall_ctx, &osmux_test_log_info);
 	log_set_log_level(osmo_stderr_target, LOGL_DEBUG);
 
 	osmux_xfrm_output_init(&h_output, 0x7000000);
+	osmux_xfrm_output_set_tx_cb(&h_output, tx_cb, NULL);
 
 	/* If the test takes longer than 10 seconds, abort it */
 	alarm(10);
